
!PasteUpMorph methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 1/12/2007 16:57'!
modalLockTo: aSystemWindow
	"Don't lock the world!! Lock the submorphs.
	The modal window gets opened afterwards so is OK."
	
	|lockStates|
	lockStates _ IdentityDictionary new.
	self submorphsDo: [:m |
		lockStates at: m put: m isLocked.
		m lock].
	self
		setProperty: #submorphLockStates
		toValue: lockStates! !

!PasteUpMorph methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 1/12/2007 17:01'!
modalUnlockFrom: aSystemWindow
	"Don't unlock the world!! Unlock the submorphs
	that were not originally locked."
	
	|lockStates|
	lockStates := self
		valueOfProperty: #submorphLockStates
		ifAbsent: [^self].
	self removeProperty: #submorphLockStates.
	lockStates keysAndValuesDo: [:m :locked |
		locked ifFalse: [m unlock]]! !

!Morph methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 9/11/2006 09:37'!
modalLockTo: aSystemWindow
	"Lock the receiver as a modal owner of the given window."

	self lock! !

!Morph methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 9/11/2006 09:38'!
modalUnlockFrom: aSystemWindow
	"Unlock the receiver as a modal owner of the given window."

	self unlock! !

!Morph methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'tween 3/2/2008 19:41'!
openModal: aSystemWindow
	"Open the given window locking the receiver until it is dismissed.
	Answer the system window.
	Restore the original keyboard focus when closed."

	|area mySysWin keyboardFocus|
	keyboardFocus _ self activeHand keyboardFocus.
	mySysWin := self isSystemWindow ifTrue: [self] ifFalse: [self ownerThatIsA: SystemWindow].
	mySysWin ifNil: [mySysWin := self].
	mySysWin modalLockTo: aSystemWindow.
	( RealEstateAgent respondsTo: #reduceByFlaps: )
		ifTrue:[
			area := RealEstateAgent reduceByFlaps: RealEstateAgent maximumUsableArea]
		ifFalse:[
			area := RealEstateAgent maximumUsableArea].
	aSystemWindow extent: aSystemWindow initialExtent.
	aSystemWindow position = (0@0)
		ifTrue: [aSystemWindow
				position: self activeHand position - (aSystemWindow extent // 2)].
	aSystemWindow
		bounds: (aSystemWindow bounds translatedToBeWithin: area).
	[ToolBuilder default runModal: aSystemWindow openAsIs]
		ensure: [mySysWin modalUnlockFrom: aSystemWindow.
				self activeHand newKeyboardFocus: keyboardFocus].
	^aSystemWindow! !

!SystemWindow methodsFor: '*PinesoftEnhancementsForFreetype-override' stamp: 'gvc 9/21/2007 14:42'!
activate
	"Activate the owner too."

	|mo mc|
	mo := self modalOwner.
	mc := self modalChild.
	mc isNil
		ifFalse: [mc owner notNil ifTrue: [
				mc activate.
				^mc modalChild isNil ifTrue: [mc flash]]].
	(self paneMorphs size > 1 and: [self splitters isEmpty])
		ifTrue: [self addPaneSplitters].
	self basicActivate.
	self rememberedKeyboardFocus
		ifNil: [(self respondsTo: #navigateFocusForward)
				ifTrue: [self navigateFocusForward]]
		ifNotNilDo: [:m | m world
						ifNil: [self rememberKeyboardFocus: nil] "deleted"
						ifNotNilDo: [:w | 
							m wantsKeyboardFocus
								ifTrue: [m takeKeyboardFocus]
								ifFalse: [(self respondsTo: #navigateFocusForward)
											ifTrue: [self navigateFocusForward]]]].
	(mo notNil and: [mo isKindOf: SystemWindow])
		ifTrue: [mo bringBehind: self]! !

!SystemWindow methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 2/5/2007 11:35'!
basicActivate
	"Bring me to the front and make me able to respond to mouse and keyboard.
	Was #activate (sw 5/18/2001 23:20)"

	| oldTop outerMorph sketchEditor pal |
	outerMorph := self topRendererOrSelf.
	outerMorph owner ifNil: [^ self "avoid spurious activate when drop in trash"].
	oldTop := TopWindow.
	oldTop = self ifTrue: [^self].
	TopWindow := self.
	oldTop ifNotNil: [oldTop passivate].
	outerMorph owner firstSubmorph == outerMorph
		ifFalse: ["Bring me (with any flex) to the top if not already"
				outerMorph owner addMorphFront: outerMorph].
	self submorphsDo: [:m | m unlock].
	labelArea ifNotNil:
		[labelArea submorphsDo: [:m | m unlock].
		self setStripeColorsFrom: self paneColorToUse].
	self isCollapsed ifFalse:
		[model modelWakeUpIn: self.
		self positionSubmorphs.
		labelArea ifNil: [self adjustBorderUponActivationWhenLabeless]].

	(sketchEditor := self extantSketchEditor) ifNotNil:
		[sketchEditor comeToFront.
		(pal := self world findA: PaintBoxMorph) ifNotNil:
			[pal comeToFront]].
! !

!SystemWindow methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 1/16/2007 12:51'!
bringBehind: aMorph
	"Make the receiver be directly behind the given morph.
	Take into account any modal owner and propagate."

	|outerMorph|
	outerMorph := self topRendererOrSelf.
	outerMorph owner ifNil: [^ self "avoid spurious activate when drop in trash"].
	outerMorph owner addMorph: outerMorph after: aMorph topRendererOrSelf.
	self modalOwner ifNotNilDo: [:mo | mo bringBehind: self]! !

!SystemWindow methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 1/16/2007 12:24'!
modalChild
	"Answer the modal child of the receiver, if any."

	^self valueOfProperty: #modalChild! !

!SystemWindow methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 1/16/2007 12:27'!
modalLockTo: aSystemWindow
	"Lock the receiver as a modal owner of the given window."

	aSystemWindow
		setProperty: #modalOwner toValue: self.
	self setProperty: #modalChild toValue: aSystemWindow! !

!SystemWindow methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 1/16/2007 12:24'!
modalOwner
	"Answer the modal owner of the receiver, if any."

	^self valueOfProperty: #modalOwner! !

!SystemWindow methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 1/16/2007 12:26'!
modalUnlockFrom: aSystemWindow
	"Unlock the receiver as a modal owner of the given window."

	aSystemWindow removeProperty: #modalOwner.
	self removeProperty: #modalChild.
	self activate! !

!SystemWindow methodsFor: '*PinesoftEnhancementsForFreetype' stamp: 'gvc 9/4/2007 13:42'!
rememberedKeyboardFocus
	"Answer the remembered keyboard focus for the receiver."
	
	^self valueOfProperty: #rememberedFocus! !
